import os

HOME_DIRECTORY = '/openhub/'
SRC_DIRECTORY = HOME_DIRECTORY + 'src/'
CONFIG = HOME_DIRECTORY + 'config/'
HARDWARE_CONFIG_FILE = CONFIG + 'hardware/config.txt'
HOMEKIT_CONFIG_FILE = CONFIG + 'homekit_accessories/config.txt'
CALIBRATION_CONFIG_FILE = CONFIG + 'calibration/config.txt'


# HAP_PYTHON_CHARACTERISTICS_FILE="""/home/lesserdaemon/.hap-python/OpenHub/characteristics.json"""
HAP_PYTHON_CHARACTERISTICS_FILE=os.path.dirname(__file__) + """/characteristics.json"""
# HAP_PYTHON_SERVICES_FILE="""/home/lesserdaemon/.hap-python/OpenHub/services.json"""
HAP_PYTHON_SERVICES_FILE=os.path.dirname(__file__)+"""/services.json"""


HAP_PYTHON_ACCESSORIES_FILE="""~/accessory.state"""
# HAP_PYTHON_ACCESSORIES_FILE='/home/lesserdaemon/.hap-python/OpenHub/accessory.state'

# HARDWARE_CONFIG_FILE_OLD="""/home/lesserdaemon/.hap-python/OpenHub/soil_sensor.json"""


import logging

from hardware_interfaces.base_io.base_io_interface import BaseIOInterface


class PiPicoPWM(BaseIOInterface):
    logger = logging.getLogger(__name__)

    def __init__(self, pwm_pin=1, duty_min=0, duty_max=65025, freq=100, *args, **kwargs):
        self.pwm_pin = pwm_pin
        self.duty_min = duty_min
        self.duty_max = duty_max
        self.freq =freq
        super(PiPicoPWM, self).__init__(*args, **kwargs)



import json
from calibrators import RawValueConverter


class RawValueEncoder(json.JSONEncoder):

    def default(self, o):
        if isinstance(o, RawValueConverter):
            dict = o.__dict__
            dict['type'] = o.__name__

        return json.JSONEncoder.default(self, o)



import json

from OpenHub.globals import id_hardware_map

from hardware_interfaces.base_io.base_io_interface import BaseIOInterface

import logging


class PiPicoAnalogIO(BaseIOInterface):

    def __init__(self, config, hardware_serial_no=None, serial_no=None, *args, **kwargs):
        self.pipico = id_hardware_map[config['hardware']]
        self.channel_index = config['channel_index']


    async def get_raw_data(self):
        command = "data" + str(self.channel_index)
        try:
            response = await self.pipico.send_command(command)
            parsed_response = response.decode('utf8').replace("'", '"')
            sensor_response = json.loads(parsed_response)
        except Exception as e:
            self.logger.log(level=logging.WARN,msg=str(e))
            return {}
        return sensor_response



# import adafruit_dht
import logging
from OpenHub.hardware_interfaces import hardware_interface
from OpenHub.hardware_interfaces.channels.dht22_humidity import DHT22Humidity
from OpenHub.hardware_interfaces.channels.dht22_temp import DHT22Temp


class DHT22(hardware_interface.HardwareInterface):
    logger = logging.getLogger(__name__)

    def __init__(self, pin, serial_no=None, channels=None, *args, **kwargs):
        # self.dhtDevice = adafruit_dht.DHT22(pin)
        self.type = self.__name__
        super().__init__(serial_no, channels, *args, **kwargs)

    def create_channel(self):
        self.create_channel_temp()
        self.create_channel_humidity()

    def create_channel_temp(self):
        self.logger.info('Creating DHT22 Temp Channel: ')
        self.channels["temp"] = DHT22Temp(hardware_serial_no=self.serial_no, dht_device=self.dhtDevice)

    def create_channel_humidity(self):
        self.logger.info('Creating DHT22 Humidity Channel: ')
        self.channels["humidity"] = DHT22Humidity(hardware_serial_no=self.serial_no, dht_device=self.dhtDevice)

    def get_channel(self, channel):
        return self.channels[channel]
    #
    # def get_temp_c(self):
    #     try:
    #         return float(self.dhtDevice.temperature)
    #     except RuntimeError as error:
    #         # Errors happen fairly often, DHT's are hard to read, just keep going
    #         self.logger.info(error.args[0])
    #         time.sleep(2.0)
    #         self.get_temp_c()
    #     except Exception as error:
    #         self.dhtDevice.exit()
    #         self.logger.error("DHT Device is down", error)
    #
    # def get_temp_f(self):
    #     temp_c = self.get_temp_c()
    #     return float(temp_c * (9.0 / 5.0) + 32.0)
    #
    # def get_humidity(self):
    #     try:
    #         return float(self.dhtDevice.humidity)
    #     except RuntimeError as error:
    #         # Errors happen fairly often, DHT's are hard to read, just keep going
    #         self.logger.info(error.args[0])
    #         time.sleep(2.0)
    #         self.get_humidity()
    #     except Exception as error:
    #         self.dhtDevice.exit()
    #         self.logger.error("DHT Device is down", error)



import logging
from OpenHub.homekit_accessories.homkit_sensor_interface import HomeKitSensorInterface


class SoilTemperatureSensor(HomeKitSensorInterface):
    logger = logging.getLogger(__name__)
    index = None
    run_debug_message = "Current Soil Temperature: "

    def __init__(self, serial_no=None, display_name=None, channel_interface_serial_no=None, *args, **kwargs):
        self.from_json(kwargs["data"])
        if self.display_name is None:
            self.display_name = display_name + "SoilTemperature"
        super().__init__(serial_no=serial_no, display_name=display_name,
                         channel_interface_serial_no=channel_interface_serial_no, *args, **kwargs)

    def set_display_name(self, display_name):
        if self.display_name is None:
            return display_name + "SoilTemperature"

    def add_functional_service(self):
        return self.add_preload_service("TemperatureSensor")

    def add_functional_service_characteristic(self):
        return self.service.get_characteristic('CurrentTemperature')

    # async def run(self):
    #     try:
    #         self.char.set_value(modProbe.read_temp_c())
    #     except IndexError as e:
    #         self.logger.warning(str(e))



from abc import ABC, abstractmethod
import uuid


class ChannelInterface(ABC):

    def __init__(self, hardware_serial_no=None, serial_no=uuid.uuid4(), **kwargs):
        self.serial_no = serial_no
        self.hardware_serial_no = hardware_serial_no
        # hardware_id_channels_map[str(self.serial_no)] = self

    @abstractmethod
    def get_raw_data(self):
        pass



import logging
import RPi.GPIO as GPIO
import json
from hardware_interfaces import mcp3008 as mcpi

from pyhap.const import CATEGORY_SENSOR
from pyhap.accessory import Bridge
from pyhap.accessory import Accessory

import initializers.hardware.local_setup as local_setup
from initializers.hardware import first_time_setup

import os


class GardenBridge(Bridge):
    category = CATEGORY_SENSOR
    logger = logging.getLogger(__name__)

    config_file_path = """/home/lesserdaemon/.hap-python/OpenHub/soil_sensor.json"""
    config = {}

    garden_sensors = {}
    num_sensors = 5
    display_name = None

    serial_no = None

    air_temp_hum_present = False
    soil_temp_present = False
    light_level_present = False
    soil_moisture_present = False

    soil_temp_sensor = None
    air_temp_humidity_sensor = None
    light_level_sensor = None

    @classmethod
    def _gpio_setup(_cls, pin):
        if GPIO.mode() is None:
            GPIO.setmode(GPIO.BCM)
        GPIO.setup(pin, GPIO.OUT)

    def __init__(self, driver, **kwargs):
        self.configure_hub()
        super().__init__(driver, self.display_name)
        self.configure_sensors()

    def configure_hub(self):
        if os.path.exists(self.config_file_path):
            local_setup.configure_hub_from_local(self)
        else:
            first_time_setup.configure_hub_first_time(self)
        mcpi.setup(self.num_sensors)

    def configure_sensors(self):
        if os.path.exists(self.config_file_path):
            local_setup.configure_sensors_from_local(self)
        else:
            first_time_setup.configure_sensors_first_time(self)
            self.save()

    def add_info_service(self):
        """Helper method to add the required `AccessoryInformation` service.

        Called in `__init__` to be sure that it is the first service added.
        May be overridden.
        """
        serv_info = self.driver.loader.get_service("AccessoryInformation")
        serv_info.configure_char("Name", value=self.display_name)
        serv_info.configure_char("Manufacturer", value="BellyFrito")
        serv_info.configure_char("Model", value="GardenBridge")
        serv_info.configure_char("SerialNumber", value=self.serial_no)
        self.add_service(serv_info)

    def add_garden_hub_service(self):
        serv_info = self.driver.loader.get_service("GardenHub")
        serv_info.configure_char("IsConfigured", False)

        self.add_service(serv_info)

    def calibrate_sensor(self, index):
        self.garden_sensors[index].calibrate()

    def calibrate(self):
        for key in self.garden_sensors.keys():
            self.garden_sensors[key].calibrate()

    def save(self):
        self.update_config_sensors()

        with open(self.config_file_path, 'w') as file:
            file.write(json.dumps(self.config))
            file.close()

    def update_config_sensors(self):
        self.config["sensors"] = {}
        soil_moisture_sensors = {}
        ind = 0
        for garden_sensor in self.garden_sensors.values():
            soil_moisture_sensors[ind] = garden_sensor.as_json()
            ind = ind + 1
        self.config["sensors"]["soil_moisture_sensors"] = soil_moisture_sensors
        if self.air_temp_humidity_sensor is not None:
            self.config["sensors"]["air_temperature_humidity_sensor"] = self.air_temp_humidity_sensor.as_json()
        if self.soil_temp_sensor is not None:
            self.config["sensors"]["soil_temp_sensor"] = self.soil_temp_sensor.as_json()
        if self.light_level_sensor is not None:
            self.config["sensors"]["light_level_sensor"] = self.light_level_sensor.as_json()
        if first_time_setup.picos is not None:
            self.config["sensors"]["picos"] = first_time_setup.pico_configs
        self.config["aid"] = self.aid

    @Accessory.run_at_interval(5)
    async def run(self):
        await super().run()



import json
from hardware_interfaces.json import hardware_interface_decoder
from homekit_accessories.json import homekit_decoder
from calibrators.json import raw_value_decoder
from OpenHub import HARDWARE_CONFIG_FILE, HOMEKIT_CONFIG_FILE, CALIBRATION_CONFIG_FILE


def load_hardware_config():
    global hardware
    hardware = json.load(open(HARDWARE_CONFIG_FILE), cls=hardware_interface_decoder.HardwareDecoder)


def load_homekit_accessory_config():
    global accessories
    accessories = json.load(open(HOMEKIT_CONFIG_FILE), cls=homekit_decoder.HomekitDecoder)


def load_calibration_config():
    global calibration
    calibration = json.load(open(CALIBRATION_CONFIG_FILE), cls=raw_value_decoder.RawValueDecoder)


def load_configs():
    load_hardware_config()
    load_homekit_accessory_config()
    load_calibration_config()


